# -*- coding: ISO-8859-1 -*-

##############################################################################
#
# Copyright (c) 2006 Alexis Roda. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
##############################################################################

"""\
ZipTool help
============

``portal_ziptool`` ofereix serveis (operacions) per treballar amb
arxius comprimits. Els serveis oferits es poden classificar en dos
grups:

  * serveis independents de la ubicacio dins la jerarquia d'objectes,
    per exemple el servei que permet determinar si un arxiu es valid

  * serveis dependents de la ubicacio dins la jerarquia, per exemple
    el servei per descomprimir esta influenciat pels permisos, que
    depenen del lloc.

Aquesta distincio es rellevant perque te implicacions en la seguretat,
concretament influeix on es poden definir els permisos.

Seguretat
---------

Actualment portal_ziptool defineix dos permisos, ``ziptool: query zip
services`` i ``ziptool: use unzip services``

Pels serveis dependents de la ubicacio podem assignar els permisos
alli on convingui, pero pels serveis independents de la ubicacio cal
assignar els permisos en el mateix nivell (o superior) on esta
l'objecte ``portal_ziptool``.

El serveis oferits per ``portal_ziptool`` poden requerir una gran
quantitat de memoria i temps de processador en funcio del tamany de
l'arxiu a processar. Ao el fa vulnerable a atacs de denegacio de
servei (DOS, denial of service). Per aquest motiu cal tindre cura de a
quins rols s'assignen els permisos.

API
---

ZipTool defineix les operacions seguents:

  * ``isSupportedArchive(nom, arxiu)`` :

    Esta protegit pel permis ``ziptool: query zip services``

    Si ``arxiu`` es reconegut com un arxiu comprimit valid retorna
    ``True``, sino retorna ``False``.

    Es **independent** de la ubicacio.

  * ``checkNames(nom, arxiu)`` :

    Esta protegit pel permis ``ziptool: query zip services``

    Verifica que els noms dels membres de ``arxiu`` compleixen les
    restriccions imposades per Zope. Si tot es correcte retorna
    ``None``, sino retorna una llista amb el nom dels arxius que
    violen les restriccions.

    Es **independent** de la ubicacio.

  * ``unzip(nom, arxiu, ruta_desti, onduplication='error')`` :

    Esta protegit pel permis ``ziptool: use unzip services``

    Descomprimeix l'arxiu ``arxiu`` en la ubicacio identificada per
    ``ruta_desti`` (crec que pot ser una ruta absoluta o
    relativa). Cal que l'usuari tingui el permis per descomprimir a la
    ubicacio de desti. Tambe cal que l'usuari pugui travesar fins el
    desti i hi tingui els permisos per crear els objectes.

    El parametre ``onduplication`` controla que ocorre quan es
    descomprimeix un arxiu que ja existeix i pot tindre un els valors:

      * ``error`` : dispara una excepcio

      * ``ignore`` : ignora l'arxiu, mantenint l'original

      * ``overwrite`` : reemplaa l'arxiu

    Es **dependent** de la ubicacio.

------------------------------------------------------------------------

$Id: ziptool.py 51 2006-09-30 17:35:54Z alex $
"""

__version__ = "$Revision: $"
# $Source: $

import mimetypes
import os.path

from AccessControl import ClassSecurityInfo
from AccessControl import Permissions as ZopePermissions
from AccessControl import Unauthorized
from Acquisition import aq_base
from Globals import InitializeClass
from Globals import DTMLFile
#from OFS.Folder import manage_addFolder
from OFS.Image import manage_addImage
from OFS.Image import manage_addFile
from OFS.SimpleItem import SimpleItem
from OFS.ObjectManager import bad_id
from reStructuredText import HTML

import permissions
from zipfolder import manage_addZipFolder
import archiverregistry
import compressorregistry
from exceptions import BadArchiveError
from exceptions import UnsupportedFormatError
from utils import _checkPermission


_formatted_doc = None

class ZipTool(SimpleItem) :
    """
    """

    security = ClassSecurityInfo()

    id = 'ziptool'
    meta_type = 'ziptool:ziptool'

    manage_options = ({'label'  : 'Overview',
                       'action' : 'manage_overview'},
                      {'label'  : 'Documentation',
                       'action' : 'manage_docs'},
                      ) + SimpleItem.manage_options

    manage_overview = DTMLFile('dtml/manage_overview', globals())
    manage_docs = DTMLFile('dtml/manage_docs', globals())

    security.declareProtected(permissions.query_zip_services,
                              'isSupportedArchive')
    def isSupportedArchive(self, archname, file_) :
        """
        """
        try:
            archive = self._get_archive_handler(archname, file_)
            return True
        except (UnsupportedFormatError, BadArchiveError) :
            return False

    security.declareProtected(permissions.query_zip_services,
                              'checkNames')
    def checkNames(self, archname, file_) :
        """
        """
        archive = self._get_archive_handler(archname, file_)

        errors = []
        seen = {}
        ERROR = 1
        OK = 0
        for m in archive :
            for p in m.filename().split('/') :
                if p not in seen :
                    if self._bad_id(p) :
                        seen[p] = ERROR
                    else :
                        seen[p] = OK

                if seen[p] == ERROR :
                    errors.append(m.filename())
                    break

        if errors :
            return errors
        else :
            return None

    security.declareProtected(ZopePermissions.view, 'unzip')
    def unzip(self, archname, file_, destination_path, onduplication='error') :
        """
        """
        onduplication = onduplication.lower()
        if onduplication not in ('error', 'ignore', 'overwrite') :
            raise ValueError('Invalid onduplication value: %s' % onduplication)

        dest_container = self.restrictedTraverse(destination_path)
        if not _checkPermission(permissions.use_unzip_services,
                                dest_container) :
            raise Unauthorized("You are not allowed to unzip here.")

        archive = self._get_archive_handler(archname, file_)

        for member in archive :
            dirname, filename = os.path.split(member.filename())
            folder = self._create_path(dest_container, dirname)
            if hasattr(aq_base(folder), filename) :
                if onduplication == 'error' :
                    raise ValueError('Duplicated object: %s/%s' % (dirname, filename))
                elif onduplication == 'overwrite' :
                    folder.manage_delObjects([filename])
                elif onduplication == 'ignore' :
                    continue
                else :
                    assert False, "should'nt get here"

            self._create_member(folder, filename, member)
        archive.close()

    security.declareProtected(ZopePermissions.view_management_screens,
                              'docs')
    def docs(self) :
        """
        """
        global _formatted_doc

        if _formatted_doc is None :
            _formatted_doc = HTML(__doc__)

        return _formatted_doc

    security.declarePrivate('_create_path')
    def _create_path(self, container, name) :
        if name :
            # cal comprovar-ho explicitament, ''.split('/') retorna
            # [''], no [] !!!
            for part in name.split('/') :
                if not hasattr(aq_base(container), part) :
                    self._create_folder(container, part)
                container = getattr(container, part)
        return container

    security.declarePrivate('_create_folder')
    def _create_folder(self, container, name) :
        manage_addZipFolder(container, name)

    security.declarePrivate('_create_member')
    def _create_member(self, container, name, member) :
        mimetype, encoding = mimetypes.guess_type(name)
        if mimetype is None :
            mimetype = 'application/octet-stream'
        type_, subtype = mimetype.split('/')
        if type_ == 'image' :
            manage_addImage(container, name,
                            file=member.read(),
                            content_type=mimetype)
        else:
            manage_addFile(container, name,
                           file=member.read(),
                           content_type=mimetype)

    def _get_archive_handler(self, archname, file_) :
        """
        """
        file_ = compressorregistry.decompress(file_)
        handler = archiverregistry.getHandler(file_)
        if handler is None :
            raise UnsupportedFormatError()

        try:
            archive = handler(archname, file_)
        except BadArchiveError:
            raise

        return archive

    def _bad_id(self, name) :
        """
        """
        return bad_id(name) is not None


InitializeClass(ZipTool)


def manage_addZipTool(self, REQUEST=None) :
    """'
    """
    zt = ZipTool()
    self._setObject(zt.id, zt)
    if REQUEST is not None :
        REQUEST.RESPONSE.redirect('./manage_main')
